﻿using System;
using System.Windows.Forms;

namespace TestProject
{
    /// <summary>
    /// 캘린더 편집 컨트롤
    /// </summary>
    public class CalendarEditingControl : DateTimePicker, IDataGridViewEditingControl
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 데이터 그리드 뷰
        /// </summary>
        private DataGridView dataGridView;

        /// <summary>
        /// 값 변경 여부
        /// </summary>
        private bool valueChanged = false;

        /// <summary>
        /// 행 인덱스
        /// </summary>
        private int rowIndex;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 편집 컨트롤 데이터 그리드 뷰 - EditingControlDataGridView

        /// <summary>
        /// 편집 컨트롤 데이터 그리드 뷰
        /// </summary>
        public DataGridView EditingControlDataGridView
        {
            get
            {
                return this.dataGridView;
            }
            set
            {
                this.dataGridView = value;
            }
        }

        #endregion
        #region 편집 패널 커서 - EditingPanelCursor

        /// <summary>
        /// 편집 패널 커서
        /// </summary>
        public Cursor EditingPanelCursor
        {
            get
            {
                return base.Cursor;
            }
        }

        #endregion
        #region 편집 컨트롤 행 인덱스 - EditingControlRowIndex

        /// <summary>
        /// 편집 컨트롤 행 인덱스
        /// </summary>
        public int EditingControlRowIndex
        {
            get
            {
                return this.rowIndex;
            }
            set
            {
                this.rowIndex = value;
            }
        }

        #endregion
        #region 편집 컨트롤 포맷 값 - EditingControlFormattedValue

        /// <summary>
        /// 편집 컨트롤 포맷 값
        /// </summary>
        public object EditingControlFormattedValue
        {
            get
            {
                return this.Value.ToShortDateString();
            }
            set
            {            
                if(value is string)
                {
                    try
                    {
                        this.Value = DateTime.Parse((string)value);
                    }
                    catch
                    {
                        this.Value = DateTime.Now;
                    }
                }
            }
        }

        #endregion
        #region 편집 컨트롤 값 변경 여부 - EditingControlValueChanged

        /// <summary>
        /// 편집 컨트롤 값 변경 여부
        /// </summary>
        public bool EditingControlValueChanged
        {
            get
            {
                return this.valueChanged;
            }
            set
            {
                this.valueChanged = value;
            }
        }

        #endregion
        #region 값 변경시 편집 컨트롤 재위치 여부 - RepositionEditingControlOnValueChange

        /// <summary>
        /// 값 변경시 편집 컨트롤 재위치 여부
        /// </summary>
        public bool RepositionEditingControlOnValueChange
        {
            get
            {
                return false;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - CalendarEditingControl()

        /// <summary>
        /// 생성자
        /// </summary>
        public CalendarEditingControl()
        {
            this.Format = DateTimePickerFormat.Short;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 편집 컨트롤 포맷 값 구하기 - GetEditingControlFormattedValue(context)

        /// <summary>
        /// 편집 컨트롤 포맷 값 구하기
        /// </summary>
        /// <param name="context">컨텍스트</param>
        /// <returns>편집 컨트롤 포맷 값</returns>
        public object GetEditingControlFormattedValue(DataGridViewDataErrorContexts context)
        {
            return EditingControlFormattedValue;
        }

        #endregion
        #region 셀 스타일 편집 컨트롤 적용하기 - ApplyCellStyleToEditingControl(dataGridViewCellStyle)

        /// <summary>
        /// 셀 스타일 편집 컨트롤 적용하기
        /// </summary>
        /// <param name="dataGridViewCellStyle">데이터 그리드 뷰 셀 스타일</param>
        public void ApplyCellStyleToEditingControl(DataGridViewCellStyle dataGridViewCellStyle)
        {
            this.Font                    = dataGridViewCellStyle.Font;
            this.CalendarForeColor       = dataGridViewCellStyle.ForeColor;
            this.CalendarMonthBackground = dataGridViewCellStyle.BackColor;
        }

        #endregion
        #region 편집 컨트롤 입력 키 요청 여부 구하기 - EditingControlWantsInputKey(key, dataGridViewWantsInputKey)

        /// <summary>
        /// 편집 컨트롤 입력 키 요청 여부 구하기
        /// </summary>
        /// <param name="key">키</param>
        /// <param name="dataGridViewWantsInputKey">데이터 그리드 뷰 입력 키 요청 여부</param>
        /// <returns>편집 컨트롤 입력 키 요청 여부 구하기</returns>
        public bool EditingControlWantsInputKey(Keys key, bool dataGridViewWantsInputKey)
        {
            switch(key & Keys.KeyCode)
            {
                case Keys.Left     :
                case Keys.Up       :
                case Keys.Down     :
                case Keys.Right    :
                case Keys.Home     :
                case Keys.End      :
                case Keys.PageDown :
                case Keys.PageUp   :

                    return true;

                default :

                    return !dataGridViewWantsInputKey;
            }
        }

        #endregion
        #region 편집용 편집 컨트롤 준비하기 - PrepareEditingControlForEdit(selectAll)

        /// <summary>
        /// 편집용 편집 컨트롤 준비하기
        /// </summary>
        /// <param name="selectAll">전체 선택 여부</param>
        public void PrepareEditingControlForEdit(bool selectAll)
        {
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region 값 변경시 처리하기 - OnValueChanged(e)

        /// <summary>
        /// 값 변경시 처리하기
        /// </summary>
        /// <param name="e">이벤트 인자</param>
        protected override void OnValueChanged(EventArgs e)
        {
            this.valueChanged = true;

            this.EditingControlDataGridView.NotifyCurrentCellDirty(true);

            base.OnValueChanged(e);
        }

        #endregion
    }
}